package org.openecu.boot;

import org.openecu.Globals;
import org.openecu.comm.CommunicationException;
import org.openecu.comm.ECUMonitor;
import org.openecu.comm.ECUMonitorFactory;
import org.openecu.gui.renderer.Renderer;
import org.openecu.gui.renderer.RenderException;
import org.openecu.param.Parameter;
import org.openecu.param.ParameterException;
import org.openecu.param.ParameterRegistry;

import java.util.ArrayList;
import java.util.List;

/**
 * User: tjennings
 * Date: Jun 5, 2005
 * Time: 5:29:20 PM
 */
public class Strapper {
    private ECUMonitor _m;
    private Thread _t;

    private List<Renderer> _renderers;
    private List<Parameter> _params;


    protected Strapper() {
    };

    protected void start(String[] args) {
        try {
            parseArgs(args);
            _m = ECUMonitorFactory.getMonitor();
            _m.addAllParam(_params);
            for(Renderer r : _renderers)
                r.setParams(_params);

            _t = new Thread(_m);
            _t.start();
        } catch (CommunicationException e) {
            e.printStackTrace();
            System.exit(-1);
        } catch (ParameterException e) {
            e.printStackTrace();
            System.exit(-1);
        } catch (RenderException e) {
            e.printStackTrace();
            System.exit(-1);
        }

    }

    protected void stop() {
        _m.stop();
        if(_t.isAlive())_t.interrupt();

        try {
            for(Renderer r : _renderers) r.shutDown();
        } catch(RenderException re) {
            re.printStackTrace();
        }

    }

    public void parseArgs(String[] args) throws ParameterException {
        String state = "";
        _renderers = new ArrayList<Renderer>();
        _params = new ArrayList<Parameter>();
        boolean doCSVFile = false;

        for(String s : args) {
            if (s.substring(0, 1).equals("-")) {
                state = s;
            } else {
                if (state.equals("-render")) {
                    if (s.equals("console")) _renderers.add(Renderer.consoleRenderer());
                    else if (s.equals("csvfile")) doCSVFile = true;
                } else if (state.equals("-param")) {
                    _params.add(ParameterRegistry.getParamForName(s));
                } else if (state.equals("-file") && doCSVFile) {
                    _renderers.add(Renderer.csvFileRenderer(s));
                } else {
                    System.out.println("Unsupported Argument");
                    printUsage();
                    System.exit(-1);
                }
            }
        }
    }

    private void printUsage() {
        System.out.println("Usage: org.openecu.boot.Bootstrap -render <renderer> [<renderer>] -param <parameter> [<parameter>] [-file <file>]");
    }
}