package org.openecu.comm;

import org.openecu.param.ECUParameter;
import org.openecu.param.Parameter;
import org.openecu.param.ParameterRegistry;
import org.openecu.param.ParameterException;
import org.openecu.param.special.TimeParameter;
import org.openecu.param.meta.MetaParameter;
import org.openecu.Globals;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;

/**
 * User: tjennings
 * Date: Jun 2, 2005
 * Time: 8:03:15 PM
 */
public abstract class BaseMonitor implements ECUMonitor{
    protected List<ECUParameter> _params;
    private TimeParameter _time;
    protected Boolean _doRun;

    public BaseMonitor() throws ParameterException {

        _params = Collections.synchronizedList(new ArrayList<ECUParameter>());
        _doRun = new Boolean(true);
        _time = (TimeParameter) ParameterRegistry.getParamForName(Globals.TIME_PARAM);
    }

    public void stop() {
        synchronized(_doRun) {
            _doRun = new Boolean(false);
        }
    }

    protected void markTime() {
        _time.marktime();
    }

    public void addParam(ECUParameter p) {
        if(!_params.contains(p)) _params.add(p);
    }

    public void addAllParam(List<Parameter> params) {
        List<Parameter> l = new ArrayList<Parameter>(params.size());
        l.addAll(params);
        recurseParam(l);
    }

    public void recurseParam(List<Parameter> params) {
            if(params.size() == 0) return;

            if(params.get(0) instanceof MetaParameter) {
                params.addAll( ((MetaParameter) params.get(0)).getDependants());
            } else if (params.get(0) instanceof TimeParameter) {
                /*do nothing*/
            } else {
                if(!_params.contains(params.get(0))) _params.add((ECUParameter)params.get(0));
            }
            params.remove(0);
            recurseParam(params);
    }
}
