package org.openecu.comm;

import gnu.io.RXTXPort;
import org.openecu.param.ECUParameter;
import org.openecu.param.ParameterException;
import org.openecu.util.UnsignedByte;

import java.io.InputStream;
import java.io.OutputStream;

import java.util.concurrent.TimeUnit;

/**
 * User: tjennings
 * Date: May 29, 2005
 * Time: 12:08:11 PM
 */
public class SSPMonitor extends BaseMonitor {
    public static final int SSP_HEADER_LEN = 5;
    public static final int SSP_CHECKSUM_LEN = 1;
    public static final int SSP_ADDRESS_SIZE = 3;
    public static final int SSP_HEADER_PADDING_LEN = 1;
    public static final int SSP_RESULT_DATA_OFFSET = 5;

    public SSPMonitor() throws ParameterException {
        super();
    }

    public void run() {
        RXTXPort port = null;

        try {
            port = getPort();
            port.close();  //probably unecssary.  Cleaning up stale locks.
            port = getPort();
            byte[] query = getQuery();
            byte[] result = getResultForQuery(query);
            OutputStream os = port.getOutputStream();
            InputStream is = port.getInputStream();

            if (is.available() > 0) {
                is.read(new byte[is.available()], 0, is.available());
                System.out.println("Read Stale data");
            }
            int timeout = 0;
            while(_doRun.booleanValue()) {
                port.setRTS(false);
                os.write(query, 0, query.length);
                os.flush();
                timeout = 50;
                while(is.available() < result.length) {
                    TimeUnit.MILLISECONDS.sleep(5);
                    timeout -= 5;
                    if(timeout <= 0) {
                        byte[] b = new byte[is.available()];
                        is.read(b, 0, b.length);
                        dumpBadResult(b);
                        break;
                    }
                }
                if(timeout <= 0) continue;

                int read = is.read(result, 0, result.length);
                sendResult(result, query.length);
                markTime();
            }
            os.close();
            is.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (port != null) port.close();
        }
    }

    private void sendResult(byte[] result, int queryOffset) {
        int totOffset = queryOffset + SSP_RESULT_DATA_OFFSET;
        int dataLen = result.length - SSP_CHECKSUM_LEN;
        for(int i = totOffset, x = 0; i < dataLen; i++, x++) {
            ECUParameter p = _params.get(x);
            p.setResult(new UnsignedByte(result[i]));
        }
    }

    private byte[] getResultForQuery(byte[] q) {
        int dataSize = 0;
        for(ECUParameter p : _params) {
            dataSize += p.getQueryLength();
        }
        return new byte[q.length + dataSize + SSP_HEADER_LEN + SSP_CHECKSUM_LEN];
    }

    private byte[] getQuery() {
        int dataBytes = SSP_HEADER_PADDING_LEN;

        dataBytes += SSP_ADDRESS_SIZE * _params.size();

        byte[] query = new byte[dataBytes
                                + SSP_HEADER_LEN
                                + SSP_CHECKSUM_LEN];

        //Standard header for this operation
        query[0] = (byte) 0x80;  //Don't know what this is.
        query[1] = (byte) 0x10;  //Destination: ECU;
        query[2] = (byte) 0xF0;  //Source: Diagnostic App.
        query[3] = (byte) (dataBytes + 1);
        //Command Portion, these two count as Data!
        query[4] = (byte) 0xA8;  //Command Byte: Read single Address;
        query[5] = (byte) 0x00;  //Required padding

        int i = SSP_HEADER_LEN + 1;
        for(ECUParameter p : _params) {
            byte[] q = p.getQuery();
            for(int x = 0; x < SSP_ADDRESS_SIZE; x++, i++)
                query[i] = q[x];
        }

        query[query.length - 1] = checkSum(query);
        return query;
    }

    private static byte checkSum(byte[] b) {
        int len = b.length - 1;
        byte sum = 0;
        for(int i = 0; i < len; i++) {
            sum += b[i];
        }
        return sum;
    }

    private void dumpBadResult(byte[] b) {
        System.err.print("Bad Msg: [");
        for(byte aB : b) {
            System.err.print(Byte.toString(aB) + " ");
        }
        System.err.print("]\n");
    }

    private RXTXPort getPort() {
        try {
            RXTXPort port = new gnu.io.RXTXPort(System.getProperty("monitor.port"));
            port.setFlowControlMode(RXTXPort.FLOWCONTROL_NONE);
            port.setSerialPortParams(4800, RXTXPort.DATABITS_8, RXTXPort.STOPBITS_1, RXTXPort.PARITY_NONE);
            port.disableReceiveTimeout();
            return port;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

}
