package org.openecu.gui.renderer;

import org.openecu.param.ECUParameter;
import org.openecu.param.Parameter;

import java.util.Map;
import java.util.List;
import java.util.Observer;
import java.util.Observable;
import java.io.File;

/**
 * User: tjennings
 * Date: Jun 2, 2005
 * Time: 7:34:00 PM
 */
public abstract class Renderer implements Observer {
    protected List<Parameter> _params;
    private int _count = 0;

    public static Renderer consoleRenderer() {
        return new ConsoleRenderer();
    }

    public static Renderer csvFileRenderer(String f) {
        return new CSVFileRenderer(f);
    }

    public Renderer(List<Parameter> params) {
        this._params = params;
        for(Parameter p : params) {
            p.addObserver(this);
        }
    }

    public Renderer() {

    }

    public void setParams(List<Parameter> params) throws RenderException {
        if(_params != null) detach();
        _params = params;
        for(Parameter p : _params) {
            p.addObserver(this);
        }
    }

    private void detach() {
         for(Parameter p : _params) {
            p.deleteObserver(this);
        }
    }

    protected abstract void render() throws RenderException;

    public abstract void shutDown() throws RenderException;

    public void update(Observable o, Object arg) {
        if(_count == _params.size()) {
            try {
                render();
            } catch(RenderException re) {
                re.printStackTrace();
            }
            _count = 0;
        } else {
            _count++;
        }

    }
}
