package org.openecu.param.loader;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.openecu.Globals;
import org.openecu.param.ECUParameter;
import org.openecu.param.Parameter;
import org.openecu.param.ParameterException;
import org.openecu.param.conversion.ConversionHandler;
import org.openecu.param.conversion.ssp.NullHandler;
import org.openecu.param.meta.MetaParameter;
import org.openecu.util.UnsignedByte;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * User: tjennings
 * Date: May 28, 2005
 * Time: 11:59:12 PM
 */
public class XMLParameterLoader {
    public static final String PARAM_FILE = "parameter.file";
    //XML Element names
    public static final String ROOT_PARAM = "parameters";
    public static final String PARAM = "parameter";
    public static final String ADDRESS = "address";
    public static final String BYTE = "byte";
    public static final String HANDLER = "handler";
    public static final String META_PARAM = "meta-parameter";
    public static final String ARGS = "args";
    public static final String ARG_NAME = "name";
    public static final String ARG_VALUE = "value";

    private Element _root;

    public XMLParameterLoader() throws ParameterException {
        try {
            SAXBuilder b = new SAXBuilder();
            String paramFile = System.getProperty(Globals.CONF_DIR) + '/' + System.getProperty(PARAM_FILE);
            Document d = b.build(new File(paramFile));
            _root = d.getRootElement();
        } catch (JDOMException jde) {
            throw new ParameterException(jde);
        } catch (IOException ioe) {
            throw new ParameterException(ioe);
        }
    }

    public Map<String, Parameter> loadParams() throws ParameterException {
        return parseParameters(_root.getChildren(PARAM));
    }

    public Map<String, Parameter> loadMetaParams() throws ParameterException {
        return parseMetaParameters(_root.getChildren(META_PARAM));
    }


    private Map<String, Parameter> parseParameters(List params) throws ParameterException {

        Map<String, Parameter> paramMap = new HashMap<String, Parameter>();
        Iterator i = params.iterator();

        while(i.hasNext()) {
            Element aParam = (Element) i.next();
            ConversionHandler handler = (ConversionHandler) getHandler(aParam);
            if (handler == null) handler = new NullHandler();

            ECUParameter ECUEcuP = new ECUParameter(getBytes(aParam),
                                                    aParam.getTextTrim(),
                                                    handler);
            paramMap.put(ECUEcuP.getName(), ECUEcuP);
        }
        return paramMap;

    }

    private Map<String, Parameter> parseMetaParameters(List metaParams) throws ParameterException {
        Map<String, Parameter> paramMap = new HashMap<String, Parameter>();
        Iterator i = metaParams.iterator();

        while(i.hasNext()) {
            Element aParam = (Element) i.next();
            MetaParameter mp = (MetaParameter) getHandler(aParam);
            if (mp == null) throw new ParameterException("Meta-parameter must declare a handler");
            mp.setName(aParam.getTextTrim());
            mp.setArgs(getMetaArgs(aParam));
            paramMap.put(mp.getName(), (Parameter) mp);
        }
        return paramMap;
    }

    private Map<String, String> getMetaArgs(Element aParam) {
        List args = aParam.getChild(ARGS).getChildren();
        Map<String, String> argMap = new HashMap<String, String>(args.size());
        Iterator i = args.iterator();
        while(i.hasNext()) {
            Element anArg = (Element) i.next();
            argMap.put(anArg.getAttributeValue(ARG_NAME), anArg.getAttributeValue(ARG_VALUE));
        }
        return argMap;
    }

    private UnsignedByte[] getBytes(Element aParam) throws ParameterException {
        List elements = aParam.getChild(ADDRESS).getChildren(BYTE);
        if (elements.size() != 3) throw new ParameterException("Three address bytes must be declared");

        UnsignedByte[] b = new UnsignedByte[3];
        for(int i = 0; i < 3; i++) {
            Element e = (Element) elements.get(i);
            b[i] = new UnsignedByte(e.getTextTrim());
        }

        return b;
    }

    private Object getHandler(Element aParam) throws ParameterException {
        Element handler = aParam.getChild(HANDLER);
        if (handler == null) return null;

        try {
            Object o = Class.forName(handler.getTextTrim()).newInstance();
            return o;
        } catch (InstantiationException ie) {
            throw new ParameterException(ie);
        } catch (IllegalAccessException iae) {
            throw new ParameterException(iae);
        } catch (ClassNotFoundException cnfe) {
            throw new ParameterException(cnfe);
        }
    }
}
