package org.openecu.test.comm;

import junit.framework.TestCase;
import junit.framework.Assert;

import java.util.List;
import java.util.concurrent.TimeUnit;
import java.io.OutputStream;
import java.io.InputStream;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.RXTXPort;

/**
 * User: tjennings
 * Date: May 24, 2005
 * Time: 8:51:41 PM
 */
public class comm_UT extends TestCase {
    private static byte[] msg;
    static {
        msg = new byte[]{
          new Integer(0x80).byteValue(),
          new Integer(0x10).byteValue(),
          new Integer(0xF0).byteValue(),
          new Integer(0x11).byteValue(),
          new Integer(0xA8).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x0D).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x0E).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x0F).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x11).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x00).byteValue(),
          new Integer(0x22).byteValue(),
          new Integer(0x00).byteValue(), //Space for the checksum
        };
        msg[msg.length - 1] = checkSum(msg);
    }

    private static byte checkSum(byte[] b) {
        int len = b.length - 1;
        byte sum = 0;
        for(int i = 0; i < len; i++) {
            sum+=b[i];
        }
        return sum;
    }


    public void testListCommPorts() throws Exception {
        RXTXPort port = new gnu.io.RXTXPort("/dev/ttyS0");
        port.setFlowControlMode(RXTXPort.FLOWCONTROL_NONE);
        port.setSerialPortParams(4800, RXTXPort.DATABITS_8, RXTXPort.STOPBITS_1, RXTXPort.PARITY_NONE);
        port.disableReceiveTimeout();
        //port.disableReceiveFraming();
        //port.disableReceiveThreshold();
        Assert.assertTrue(port.getFlowControlMode() == RXTXPort.FLOWCONTROL_NONE);
        Assert.assertTrue(port.getBaudRate() == 4800);
        Assert.assertTrue(port.getDataBits() == RXTXPort.DATABITS_8);
        Assert.assertTrue(port.getStopBits() == RXTXPort.STOPBITS_1);
        Assert.assertTrue(port.getParity() == RXTXPort.PARITY_NONE);

        OutputStream os = port.getOutputStream();
        InputStream is = port.getInputStream();
        if(is.available() > 0) is.read(new byte[is.available()], 0, is.available());
        byte[] input = new byte[128];

        System.out.print("SND(" + msg.length + "): ");
        for(byte t : msg) System.out.print(unsignedByteToInt(t) + " ");
        System.out.println("");

        int cnt = 0;
        while( (cnt++) < 500) {
            port.setRTS(false);
            os.write(msg, 0, 22);
            os.flush();
            while(is.available() < 33) TimeUnit.MILLISECONDS.sleep(50);

            int read = is.read(input, 0, 33);
            int map = (unsignedByteToInt(input[27])) * 37 / 255;
            int rpmH = unsignedByteToInt(input[28]);
            int rpmL = unsignedByteToInt(input[29]);
            rpmH <<= 8;
            int rpm = (rpmH + rpmL) / 4;
            int ign = (unsignedByteToInt(input[30]) - 128) / 2;
            int knk = (unsignedByteToInt(input[31]) - 128) / 2;
            System.out.print("RPM: " + rpm);
            System.out.print(" MAP: " + map);
            System.out.print(" IGN: " + ign);
            System.out.print(" KNK: " + knk);
            System.out.println("");
            //System.out.print("Read: " + read);
            //System.out.print(" RPMH: " + input[28]);
            //System.out.println();
            //dumpMsg(input);
        }

        port.close();
    }

    private void dumpMsg(byte[] m) {
        System.out.print("REC(" + m.length + ") ");
        for(byte t : m) {
            System.out.print(unsignedByteToInt(t)+ " ");
        }
        System.out.println("");
    }

    public static int unsignedByteToInt(byte b) {
        return (int) b & 0xFF;
    }

}
