package org.openecu.test;

import junit.framework.TestCase;
import junit.framework.Assert;
import org.openecu.param.ECUParameter;
import org.openecu.param.ParameterRegistry;
import org.openecu.param.Parameter;
import org.openecu.util.UnsignedByte;
import org.openecu.comm.ECUMonitorFactory;
import org.openecu.comm.ECUMonitor;
import org.openecu.gui.renderer.Renderer;
import org.openecu.Globals;

import java.util.Observable;
import java.util.Observer;
import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

/**
 * User: tjennings
 * Date: May 25, 2005
 * Time: 8:02:25 PM
 */
public class testOpenEcuLogger extends TestCase {
    public void testMapConversion() throws Exception {
        ECUParameter p = (ECUParameter) ParameterRegistry.getParamForName("MAP");
        Assert.assertTrue(p.getName().equals("MAP"));
        Assert.assertTrue(p.getQuery()[2] == (0x0D & 0xFF));
        p.setResult(new UnsignedByte("0x01"));
        Assert.assertTrue(p.getResult() == 1 * 37 / 255);

        //Test observability
        TestObserver o = new TestObserver();
        p.addObserver(o);
        p.setResult(new UnsignedByte("0x02"));

        Assert.assertTrue(o.getResult() == 2 * 37 / 255);
    }

    public void testRPMMetaParameter() throws Exception {
        ECUParameter rpmh = (ECUParameter) ParameterRegistry.getParamForName("RPMH");
        ECUParameter rpml = (ECUParameter) ParameterRegistry.getParamForName("RPML");
        Parameter rpm = ParameterRegistry.getParamForName("RPM");
    }

    public void testSSPMonitor() throws Exception {
        ECUMonitor m = ECUMonitorFactory.getMonitor();
        ECUParameter p = (ECUParameter) ParameterRegistry.getParamForName("MAP");
        TestObserver o = new TestObserver();
        p.addObserver(o);
        m.addParam(p);

        new Thread(m).start();

        int i = 0;
        while(i++ < 5) {
            //while(!p.hasChanged()) TimeUnit.MILLISECONDS.sleep(5);
            TimeUnit.MILLISECONDS.sleep(5000);

        }
        m.stop();
    }

    public void testConsoleRenderer() throws Exception {
        ECUMonitor m = ECUMonitorFactory.getMonitor();
        List<Parameter> params = new ArrayList<Parameter>();
        params.add(ParameterRegistry.getParamForName(Globals.TIME_PARAM));
        params.add(ParameterRegistry.getParamForName("MAP"));
        params.add(ParameterRegistry.getParamForName("RPM"));


        Renderer r = Renderer.consoleRenderer();
        r.setParams(params);
        m.addAllParam(params);

        new Thread(m).start();

        TimeUnit.SECONDS.sleep(20);
        m.stop();
    }


    private class TestObserver implements Observer {
        private float result;
        public void update(Observable o, Object arg) {
            result = ((ECUParameter)o).getResult();
            System.out.println("Map: " + result);
        }

        public float getResult() {
            return result;
        }
    }
}
